@@ -10,7 +10,9 @@
 
 #ifndef _BMPCODEC_H
 #define _BMPCODEC_H
-#include "gdip.h"
+
+#include "bitmap-private.h"
+//#include "codecs-private.h"
 #include "dstream.h"
 
 #define BI_RGB           0
@@ -105,17 +107,7 @@ typedef struct {
 	RGBQUAD	bmiColors[1];
 } BITMAPINFO, *PBITMAPINFO, *LPBITMAPINFO;
 
-typedef enum {
-	File,
-	DStream,
-	Memory
-} ImageSource;
-
-typedef struct {
-	BYTE* ptr;
-	int size;
-	int pos;
-} MemorySource;
+#include "gdiplus-private.h"
 
 GpStatus gdip_read_bmp_image (void *pointer, GpImage **image, ImageSource source) GDIP_INTERNAL;
 GpStatus gdip_load_bmp_image_from_file (FILE *fp, GpImage **image) GDIP_INTERNAL;
@@ -129,6 +121,6 @@ ImageCodecInfo *gdip_getcodecinfo_bmp () GDIP_INTERNAL;
 /* helper functions / shared with ICOn codec */
 GpStatus gdip_read_BITMAPINFOHEADER (void *pointer, BITMAPINFOHEADER *bmi, ImageSource source, BOOL *os2format, 
 	BOOL *upsidedown) GDIP_INTERNAL;
-int gdip_read_bmp_data (void *pointer, byte *data, int size, ImageSource source) GDIP_INTERNAL;
+int gdip_read_bmp_data (void *pointer, BYTE *data, int size, ImageSource source) GDIP_INTERNAL;
 
 #endif /* _BMPCODEC_H */
