@@ -199,7 +199,7 @@ gdip_open_curve_tangents (int terms, const GpPointF *points, int count, float te
         if (count <= 2)
                 return tangents;
 
-        for (i = 1; i < count - 1; i++) {
+        for (i = 0; i < count; i++) {
                 int r = i + 1;
                 int s = i - 1;
 
@@ -475,7 +475,7 @@ gdip_get_pattern_status (cairo_pattern_t *pat)
  * scales the coordinates. E.g. 0 scale 100 == 0, while (0 + 0.5) scale 100 == 50.
  */
 
-static BOOL
+BOOL
 gdip_is_scaled (GpGraphics *graphics)
 {
 	cairo_matrix_t matrix;
