@@ -1,7 +1,7 @@
 /*
  * hatchbrush.c
  * 
- * Copyright (C) 2004,2006 Novell, Inc. http://www.novell.com
+ * Copyright (C) 2004,2006-2007 Novell, Inc. http://www.novell.com
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
  * and associated documentation files (the "Software"), to deal in the Software without restriction, 
@@ -23,12 +23,10 @@
  *
  */
 
-
 /* FIXME - match hatchbrush use the RenderOrigin stored in graphics */
 
-#include <math.h>
-#include "gdip.h"
-#include "hatchbrush.h"
+#include "hatchbrush-private.h"
+#include "graphics-private.h"
 
 static GpStatus gdip_hatch_setup (GpGraphics *graphics, GpBrush *brush);
 static GpStatus gdip_hatch_clone (GpBrush *brush, GpBrush **clonedBrush);
@@ -44,7 +42,7 @@ static BrushClass vtable = { BrushTypeHatchFill,
 			     gdip_hatch_clone,
 			     gdip_hatch_destroy };
 
-void 
+static void 
 gdip_hatch_init (GpHatch *hatch)
 {
 	gdip_brush_init (&hatch->base, &vtable);
@@ -52,7 +50,7 @@ gdip_hatch_init (GpHatch *hatch)
 	hatch->pattern = NULL;
 }
 
-GpHatch*
+static GpHatch*
 gdip_hatch_new (void)
 {
         GpHatch *result = (GpHatch *) GdipAlloc (sizeof (GpHatch));
@@ -1312,26 +1310,21 @@ gdip_hatch_clone (GpBrush *brush, GpBrush **clonedBrush)
 GpStatus
 gdip_hatch_destroy (GpBrush *brush)
 {
-	GpHatch *hbr;
+	/* a. the NULL check for brush is done by the caller, GdipDeleteBrush */
+	/* b. brush itself is freed by the caller */
 
-	if (!brush)
-		return InvalidParameter;
-
-	hbr = (GpHatch *) brush;
+	GpHatch *hbr = (GpHatch *) brush;
 
 	if (hbr->pattern) {
 		cairo_pattern_destroy (hbr->pattern);
 		hbr->pattern = NULL;
 	}
-
-	GdipFree (brush);
-
 	return Ok;
 }
 
 // coverity[+alloc : arg-*3]
 GpStatus
-GdipCreateHatchBrush (GpHatchStyle hatchstyle, int forecolor, int backcolor, GpHatch **brush)
+GdipCreateHatchBrush (GpHatchStyle hatchstyle, ARGB forecolor, ARGB backcolor, GpHatch **brush)
 {
 	if (!brush)
 		return InvalidParameter;
@@ -1358,7 +1351,7 @@ GdipGetHatchStyle (GpHatch *brush, GpHatchStyle *hatchstyle)
 }
 
 GpStatus
-GdipGetHatchForegroundColor (GpHatch *brush, int *forecolor)
+GdipGetHatchForegroundColor (GpHatch *brush, ARGB *forecolor)
 {
 	if (!brush || !forecolor)
 		return InvalidParameter;
@@ -1368,7 +1361,7 @@ GdipGetHatchForegroundColor (GpHatch *brush, int *forecolor)
 }
 
 GpStatus
-GdipGetHatchBackgroundColor (GpHatch *brush, int *backcolor)
+GdipGetHatchBackgroundColor (GpHatch *brush, ARGB *backcolor)
 {
 	if (!brush || !backcolor)
 		return InvalidParameter;
