@@ -313,7 +313,8 @@ GdipGetImageGraphicsContext (GpImage *image, GpGraphics **graphics)
 				image->active_bitmap->width, image->active_bitmap->height, image->active_bitmap->stride);
 
 	gfx = gdip_graphics_new (surface);
-	gfx->dpi_x = gfx->dpi_y = gdip_get_display_dpi ();
+	gfx->dpi_x = image->active_bitmap->dpi_horz <= 0 ? gdip_get_display_dpi () : image->active_bitmap->dpi_horz;
+	gfx->dpi_y = image->active_bitmap->dpi_vert <= 0 ? gdip_get_display_dpi () : image->active_bitmap->dpi_vert;
 	cairo_surface_destroy (surface);
 
 	gfx->image = image;
@@ -380,6 +381,8 @@ GdipDrawImageRect (GpGraphics *graphics, GpImage *image, float x, float y, float
 	BOOL need_scaling = FALSE;
 	double scaled_width, scaled_height;
 	cairo_matrix_t orig_matrix;
+	BYTE *premul = NULL;
+	cairo_surface_t *original = NULL;
 
 	if (!graphics || !image)
 		return InvalidParameter;
@@ -429,6 +432,20 @@ GdipDrawImageRect (GpGraphics *graphics, GpImage *image, float x, float y, float
 
 	/* Create a surface for this bitmap if one doesn't exist */
 	gdip_bitmap_ensure_surface (image);
+	
+	if (graphics->type != gtMemoryBitmap &&
+		gdip_bitmap_format_needs_premultiplication (image)) {
+		premul = gdip_bitmap_get_premultiplied_scan0 (image);
+		if (premul) {
+			BitmapData *data = image->active_bitmap;
+			original = cairo_image_surface_create_for_data (premul, CAIRO_FORMAT_ARGB32,
+															data->width, data->height, data->stride);
+		}
+	}
+	
+	/* if premul isn't required (or couldn't be computed, e.g. out of memory) */
+	if (!original)
+		original = image->surface;
 
 	if (width != image->active_bitmap->width || height != image->active_bitmap->height) {
 		scaled_width = (double) width / image->active_bitmap->width;
@@ -436,7 +453,8 @@ GdipDrawImageRect (GpGraphics *graphics, GpImage *image, float x, float y, float
 		need_scaling = TRUE;
 	}
 
-	pattern = cairo_pattern_create_for_surface (image->surface);
+	/* Use the original as a pattern */
+	pattern = cairo_pattern_create_for_surface (original);
 
 	cairo_pattern_set_filter (pattern, gdip_get_cairo_filter (graphics->interpolation));
 
@@ -458,6 +476,11 @@ GdipDrawImageRect (GpGraphics *graphics, GpImage *image, float x, float y, float
 	cairo_pattern_destroy (org_pattern);
 	cairo_pattern_destroy (pattern);
 
+	if (premul) {
+		cairo_surface_destroy (original);
+		GdipFree (premul);
+	}
+
 	return Ok;
 }
 
@@ -471,6 +494,8 @@ GdipDrawImagePoints (GpGraphics *graphics, GpImage *image, GDIPCONST GpPointF *d
 	cairo_matrix_t orig_matrix;
 	GpRectF tRect;
 	MetafilePlayContext *metacontext = NULL;
+	BYTE *premul = NULL;
+	cairo_surface_t *original = NULL;
 	
 	if (!graphics || !image || !dstPoints || (count != 3))
 		return InvalidParameter;
@@ -516,7 +541,21 @@ GdipDrawImagePoints (GpGraphics *graphics, GpImage *image, GDIPCONST GpPointF *d
 	/* Create a surface for this bitmap if one doesn't exist */
 	gdip_bitmap_ensure_surface (image);
 
-	pattern = cairo_pattern_create_for_surface (image->surface);
+	if (graphics->type != gtMemoryBitmap &&
+		gdip_bitmap_format_needs_premultiplication (image)) {
+		premul = gdip_bitmap_get_premultiplied_scan0 (image);
+		if (premul) {
+			BitmapData *data = image->active_bitmap;
+			original = cairo_image_surface_create_for_data (premul, CAIRO_FORMAT_ARGB32,
+															data->width, data->height, data->stride);
+		}
+	}
+	
+	/* if premul isn't required (or couldn't be computed, e.g. out of memory) */
+	if (!original)
+		original = image->surface;
+
+	pattern = cairo_pattern_create_for_surface (original);
 	cairo_pattern_set_filter (pattern, gdip_get_cairo_filter (graphics->interpolation));
 
 	org_pattern = cairo_get_source(graphics->ct);
@@ -534,6 +573,11 @@ GdipDrawImagePoints (GpGraphics *graphics, GpImage *image, GDIPCONST GpPointF *d
 	cairo_pattern_destroy (org_pattern);
 	cairo_pattern_destroy (pattern);
 
+	if (premul) {
+		cairo_surface_destroy (original);
+		GdipFree (premul);
+	}
+
 	return Ok;
 }
 
@@ -584,6 +628,8 @@ GdipDrawImageRectRect (GpGraphics *graphics, GpImage *image,
 	void		*org;
 	int		org_format;
 	BOOL		allocated = FALSE;
+	BYTE			*premul = NULL;
+	cairo_surface_t	*original = NULL;
 	
 	if (!graphics || !image)
 		return InvalidParameter;
@@ -625,14 +671,14 @@ GdipDrawImageRectRect (GpGraphics *graphics, GpImage *image,
 
 	/* see OPTIMIZE_CONVERSION in general.h */
 	if ((srcUnit != UnitPixel) && (srcUnit != UnitWorld) && ((srcUnit != UnitDisplay) || (graphics->type != gtPostScript))) {
-		dstx = gdip_unit_conversion (srcUnit, UnitCairoPoint, graphics->type, graphics->dpi_x, dstx);
-		dsty = gdip_unit_conversion (srcUnit, UnitCairoPoint, graphics->type, graphics->dpi_y, dsty);
-		dstwidth = gdip_unit_conversion (srcUnit, UnitCairoPoint, graphics->type, graphics->dpi_x, dstwidth);
-		dstheight = gdip_unit_conversion (srcUnit, UnitCairoPoint, graphics->type, graphics->dpi_y, dstheight);
-		srcx = gdip_unit_conversion (srcUnit, UnitCairoPoint, graphics->type, graphics->dpi_x, srcx);
-		srcy = gdip_unit_conversion (srcUnit, UnitCairoPoint, graphics->type, graphics->dpi_y, srcy);
-		srcwidth = gdip_unit_conversion (srcUnit, UnitCairoPoint, graphics->type, graphics->dpi_x, srcwidth);
-		srcheight = gdip_unit_conversion (srcUnit, UnitCairoPoint, graphics->type, graphics->dpi_y, srcheight);
+		dstx = gdip_unit_conversion (srcUnit, UnitCairoPoint, graphics->dpi_x, graphics->type, dstx);
+		dsty = gdip_unit_conversion (srcUnit, UnitCairoPoint, graphics->dpi_y, graphics->type, dsty);
+		dstwidth = gdip_unit_conversion (srcUnit, UnitCairoPoint, graphics->dpi_x, graphics->type, dstwidth);
+		dstheight = gdip_unit_conversion (srcUnit, UnitCairoPoint, graphics->dpi_y, graphics->type, dstheight);
+		srcx = gdip_unit_conversion (srcUnit, UnitCairoPoint, graphics->dpi_x, graphics->type, srcx);
+		srcy = gdip_unit_conversion (srcUnit, UnitCairoPoint, graphics->dpi_y, graphics->type, srcy);
+		srcwidth = gdip_unit_conversion (srcUnit, UnitCairoPoint, graphics->dpi_x, graphics->type, srcwidth);
+		srcheight = gdip_unit_conversion (srcUnit, UnitCairoPoint, graphics->dpi_y, graphics->type, srcheight);
 	}
 
 	org = dest = image->active_bitmap->scan0; 
@@ -659,14 +705,22 @@ GdipDrawImageRectRect (GpGraphics *graphics, GpImage *image,
 		BOOL		flipYOn = (imageAttributes->wrapmode == WrapModeTileFlipY);
 		BOOL		flipX = FALSE;
 		BOOL		flipY = FALSE;
-		GpBitmap	*cur_image;
 		GpBitmap	*imgflipX = NULL;
 		GpBitmap	*imgflipY = NULL;
 		GpBitmap	*imgflipXY = NULL;
 
 		float img_width = image->active_bitmap->width *  (dstwidth / srcwidth);
 		float img_height = image->active_bitmap->height * (dstheight / srcheight);
-		
+		BYTE		*premul = NULL;
+		BYTE		*premulX = NULL;
+		BYTE		*premulY = NULL;
+		BYTE		*premulXY = NULL;
+		cairo_surface_t	*original = NULL;
+		cairo_surface_t	*originalX = NULL;
+		cairo_surface_t	*originalY = NULL;
+		cairo_surface_t	*originalXY = NULL;
+		cairo_surface_t *cur_surface;
+
 		if (imageAttributes->wrapmode == WrapModeTileFlipXY) {
 			flipXOn = flipYOn = TRUE;
 		}
@@ -677,12 +731,38 @@ GdipDrawImageRectRect (GpGraphics *graphics, GpImage *image,
 			gdip_bitmap_clone (image, &imgflipX);
 			gdip_flip_x (imgflipX);	
 			gdip_bitmap_ensure_surface (imgflipX);			
+
+			if (graphics->type != gtMemoryBitmap &&
+				gdip_bitmap_format_needs_premultiplication (imgflipX)) {
+				premulX = gdip_bitmap_get_premultiplied_scan0 (imgflipX);
+				if (premulX) {
+					BitmapData *data = imgflipX->active_bitmap;
+					originalX = cairo_image_surface_create_for_data (premulX, CAIRO_FORMAT_ARGB32, data->width, data->height, data->stride);
+				}
+			}
+
+			/* if premul isn't required (or couldn't be computed, e.g. out of memory) */
+			if (!originalX)
+				originalX = imgflipX->surface;
 		}
 		
 		if (flipYOn) {			
 			gdip_bitmap_clone (image, &imgflipY);
 			gdip_flip_y (imgflipY);	
 			gdip_bitmap_ensure_surface (imgflipY);			
+			
+			if (graphics->type != gtMemoryBitmap &&
+				gdip_bitmap_format_needs_premultiplication (imgflipY)) {
+				premulY = gdip_bitmap_get_premultiplied_scan0 (imgflipY);
+				if (premulY) {
+					BitmapData *data = imgflipY->active_bitmap;
+					originalY = cairo_image_surface_create_for_data (premulY, CAIRO_FORMAT_ARGB32, data->width, data->height, data->stride);
+				}
+			}
+
+			/* if premul isn't required (or couldn't be computed, e.g. out of memory) */
+			if (!originalY)
+				originalY = imgflipY->surface;
 		}
 		
 		if (flipXOn && flipYOn) {			
@@ -690,30 +770,58 @@ GdipDrawImageRectRect (GpGraphics *graphics, GpImage *image,
 			gdip_flip_x (imgflipXY);	
 			gdip_flip_y (imgflipXY);	
 			gdip_bitmap_ensure_surface (imgflipXY);			
+
+			if (graphics->type != gtMemoryBitmap &&
+				gdip_bitmap_format_needs_premultiplication (imgflipXY)) {
+				premulXY = gdip_bitmap_get_premultiplied_scan0 (imgflipXY);
+				if (premulXY) {
+					BitmapData *data = imgflipXY->active_bitmap;
+					originalXY = cairo_image_surface_create_for_data (premulXY, CAIRO_FORMAT_ARGB32, data->width, data->height, data->stride);
+				}
+			}
+
+			/* if premul isn't required (or couldn't be computed, e.g. out of memory) */
+			if (!originalXY)
+				originalXY = imgflipXY->surface;
 		}
 		
 		gdip_bitmap_ensure_surface (image);
 
+		if (graphics->type != gtMemoryBitmap &&
+			gdip_bitmap_format_needs_premultiplication (image)) {
+			premul = gdip_bitmap_get_premultiplied_scan0 (image);
+			if (premul) {
+				BitmapData *data = image->active_bitmap;
+				original = cairo_image_surface_create_for_data (premul, CAIRO_FORMAT_ARGB32, data->width, data->height, data->stride);
+			}
+		}
+
+		/* if premul isn't required (or couldn't be computed, e.g. out of memory) */
+		if (!original)
+			original = image->surface;
+		
+
 		for (posy = 0; posy < dstheight; posy += img_height) {
 			for (posx = 0; posx < dstwidth; posx += img_width) {
 				if (flipX && flipY) {
-					cur_image = imgflipXY;
+					cur_surface = originalXY;
 				} else {
 					if (flipX) {
-						cur_image = imgflipX;
+						cur_surface = originalX;
 					} else {
 						if (flipY) {
-							cur_image = imgflipY;
+							cur_surface = originalY;
 						} else {
-							cur_image = image;
+							cur_surface = original;
 						}
 					}
 				}
 
+				cairo_matrix_translate (&mat, srcx, srcy);
 				cairo_matrix_scale (&mat, srcwidth / dstwidth, srcheight / dstheight);
-				cairo_matrix_translate (&mat, srcx - (dstx + posx), srcy - (dsty + posy));
+				cairo_matrix_translate (&mat, - (dstx + posx), - (dsty + posy));
 
-				pattern = cairo_pattern_create_for_surface(cur_image->surface);
+				pattern = cairo_pattern_create_for_surface(cur_surface);
 				cairo_pattern_set_matrix (pattern, &mat);
 
 				orig = cairo_get_source(graphics->ct);
@@ -744,28 +852,61 @@ GdipDrawImageRectRect (GpGraphics *graphics, GpImage *image,
 		
 		if (imgflipX) {
 			GdipDisposeImage ((GpImage *) imgflipX);
+			if (premulX) {
+				cairo_surface_destroy (originalX);
+				GdipFree (premulX);
+			}
 		}
 			
 		if (imgflipY) {
 			GdipDisposeImage ((GpImage *) imgflipY);
+			if (premulY) {
+				cairo_surface_destroy (originalY);
+				GdipFree (premulY);
+			}
 		}
 
 		if (imgflipXY) {
 			GdipDisposeImage ((GpImage *) imgflipXY);
+			if (premulXY) {
+				cairo_surface_destroy (originalXY);
+				GdipFree (premulXY);
+			}
+		}
+
+		if (premul) {
+			cairo_surface_destroy (original);
+			GdipFree (premul);
 		}
 	} else {
 		cairo_pattern_t *filter;
 
 		gdip_bitmap_ensure_surface (image);
-		filter = cairo_pattern_create_for_surface (image->surface);
+
+		if (graphics->type != gtMemoryBitmap &&
+			gdip_bitmap_format_needs_premultiplication (image)) {
+			premul = gdip_bitmap_get_premultiplied_scan0 (image);
+			if (premul) {
+				BitmapData *data = image->active_bitmap;
+				original = cairo_image_surface_create_for_data (premul, CAIRO_FORMAT_ARGB32, data->width, data->height, data->stride);
+			}
+		}
+	
+		/* if premul isn't required (or couldn't be computed, e.g. out of memory) */
+		if (!original)
+			original = image->surface;
+
+		filter = cairo_pattern_create_for_surface (original);
 		cairo_pattern_set_filter (filter, gdip_get_cairo_filter (graphics->interpolation));
 
+		cairo_matrix_translate (&mat, srcx, srcy);
+
 		if (!gdip_near_zero(srcwidth - dstwidth) || !gdip_near_zero(srcheight - dstheight))
 			cairo_matrix_scale (&mat, srcwidth / dstwidth, srcheight / dstheight);
 
-		cairo_matrix_translate (&mat, srcx - dstx, srcy - dsty);
+		cairo_matrix_translate (&mat, -dstx, -dsty);
 
-		pattern = cairo_pattern_create_for_surface(image->surface);
+		pattern = cairo_pattern_create_for_surface(original);
 		cairo_pattern_set_matrix (pattern, &mat);
 
 		orig = cairo_get_source(graphics->ct);
@@ -782,6 +923,11 @@ GdipDrawImageRectRect (GpGraphics *graphics, GpImage *image,
 		cairo_pattern_set_matrix (pattern, &mat);
 		cairo_pattern_destroy (pattern);
 		cairo_pattern_destroy (filter);
+
+		if (premul) {
+			cairo_surface_destroy (original);
+			GdipFree (premul);
+		}
 	}
 
 	/* The current surface is no longer valid if we had attributes applied */
@@ -1551,7 +1697,7 @@ gdip_rotate_flip_packed_indexed (GpImage *image, PixelFormat pixel_format, int a
 		int target_y_offset = target_starts_at_bottom ? (target_height - 1) * target_stride : 0;
 		int target_y_offset_delta = target_starts_at_bottom ? -target_stride : +target_stride;
 
-		Status status = gdip_init_pixel_stream (&stream, image->active_bitmap, 0, 0, image->active_bitmap->width, image->active_bitmap->height);
+		GpStatus status = gdip_init_pixel_stream (&stream, image->active_bitmap, 0, 0, image->active_bitmap->width, image->active_bitmap->height);
 
 		if (status != Ok) {
 			GdipFree (rotated);
@@ -1606,7 +1752,7 @@ gdip_rotate_flip_packed_indexed (GpImage *image, PixelFormat pixel_format, int a
 				scan_valid[i] = ((scan_index >= 0) && (scan_index < source_height));
 
 				if (scan_valid[i]) {
-					Status status = gdip_init_pixel_stream (&scan[i], image->active_bitmap, 0, scan_index, source_width, 1);
+					GpStatus status = gdip_init_pixel_stream (&scan[i], image->active_bitmap, 0, scan_index, source_width, 1);
 
 					if (status != Ok) {
 						GdipFree (rotated);
