@@ -2,6 +2,7 @@
  * solidbrush.c
  * 
  * Copyright (c) 2003 Alexandre Pigolkine
+ * Copyright (C) 2007 Novell, Inc. http://www.novell.com
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
  * and associated documentation files (the "Software"), to deal in the Software without restriction, 
@@ -23,8 +24,8 @@
  *   Ravindra (rkumar@novell.com)
  */
 
-#include "gdip.h"
-#include "solidbrush.h"
+#include "solidbrush-private.h"
+#include "graphics-private.h"
 
 static GpStatus gdip_solidfill_setup (GpGraphics *graphics, GpBrush *brush);
 static GpStatus gdip_solidfill_clone (GpBrush *brush, GpBrush **clonedBrush);
@@ -40,14 +41,14 @@ static BrushClass vtable = { BrushTypeSolidColor,
 			     gdip_solidfill_clone,
 			     gdip_solidfill_destroy };
 
-void 
+static void 
 gdip_solidfill_init (GpSolidFill *brush)
 {
 	gdip_brush_init (&brush->base, &vtable);
 	brush->color = 0;
 }
 
-GpSolidFill *
+static GpSolidFill*
 gdip_solidfill_new (void)
 {
 	GpSolidFill *result = (GpSolidFill *) GdipAlloc (sizeof (GpSolidFill));
@@ -122,16 +123,14 @@ gdip_solidfill_clone (GpBrush *brush, GpBrush **clonedBrush)
 GpStatus
 gdip_solidfill_destroy (GpBrush *brush)
 {
-	/* the NULL check for brush is done by the caller, GdipDeleteBrush */
-
-	GdipFree (brush);
-
+	/* a. the NULL check for brush is done by the caller, GdipDeleteBrush */
+	/* b. brush itself is freed by the caller */
 	return Ok;
 }
 
 // coverity[+alloc : arg-*1]
 GpStatus 
-GdipCreateSolidFill (int color, GpSolidFill **brush)
+GdipCreateSolidFill (ARGB color, GpSolidFill **brush)
 {
 	if (!brush)
 		return InvalidParameter;
@@ -145,7 +144,7 @@ GdipCreateSolidFill (int color, GpSolidFill **brush)
 }
 
 GpStatus
-GdipSetSolidFillColor (GpSolidFill *brush, int color)
+GdipSetSolidFillColor (GpSolidFill *brush, ARGB color)
 {
 	if (!brush)
 		return InvalidParameter;
@@ -157,7 +156,7 @@ GdipSetSolidFillColor (GpSolidFill *brush, int color)
 }
 
 GpStatus
-GdipGetSolidFillColor (GpSolidFill *brush, int *color)
+GdipGetSolidFillColor (GpSolidFill *brush, ARGB *color)
 {
 	if (!brush || !color)
 		return InvalidParameter;
