@@ -27,6 +27,8 @@
 
 #ifndef USE_PANGO_RENDERING
 
+#include <ctype.h>
+
 #include "text-cairo-private.h"
 #include "graphics-private.h"
 #include "graphics-cairo-private.h"
@@ -235,7 +237,7 @@ MeasureString (GpGraphics *graphics, GDIPCONST WCHAR *stringUnicode, int *length
 
 	/* unless specified we don't consider the trailing spaces, unless there is just one space (#80680) */
 	if ((format->formatFlags & StringFormatFlagsMeasureTrailingSpaces) == 0) {
-		while ((StringLen > 0) && (isspace (*(Src + StringLen - 1))))
+		while ((StringLen > 0) && (isspace ((int) ((unsigned char) *(Src + StringLen - 1)))))
 			StringLen--;
 		if (StringLen == 0)
 			StringLen = 1;
@@ -1122,6 +1124,20 @@ cairo_MeasureCharacterRanges (GpGraphics *graphics, GDIPCONST WCHAR *stringUnico
 		layoutRect->Height = gdip_unity_convgr (graphics, layout->Height);
 	}
 
+	if (layoutRect->Width <= 0.0) {
+		if (layoutRect->Height < 0.0) {
+			/* special case only if BOTH values are negative */
+			for (i = 0; i < format->charRangeCount; i++)
+				GdipSetInfinite (regions [i]);
+			return Ok;
+		} else {
+			layoutRect->Width = REGION_INFINITE_LENGTH;
+		}
+	}
+	if (layoutRect->Height <= 0.0) {
+		layoutRect->Height = REGION_INFINITE_LENGTH;
+	}
+
 	/* string measurements */
 	status = MeasureString (graphics, stringUnicode, &StringLen, font, layoutRect, format, NULL, NULL, NULL, NULL,
 		CleanString, StringDetails, &data);
