@@ -23,6 +23,7 @@
 #include "wmfcodec.h"
 
 /* Codecinfo related data*/
+GUID gdip_wmf_image_format_guid = {0xb96b3cadU, 0x0728U, 0x11d3U, {0x9d, 0x7b, 0x00, 0x00, 0xf8, 0x1e, 0xf3, 0x2e}};
 static ImageCodecInfo wmf_codec;
 static const WCHAR wmf_codecname[] = {'B', 'u', 'i','l', 't', '-','i', 'n', ' ', 'W', 'M', 'F', 0}; /* Built-in WMF */
 static const WCHAR wmf_extension[] = {'*','.','W', 'M', 'F', 0}; /* *.WMF */
@@ -49,7 +50,7 @@ gdip_getcodecinfo_wmf ()
 	wmf_codec.FormatDescription = (const WCHAR*) wmf_format;
 	wmf_codec.FilenameExtension = (const WCHAR*) wmf_extension;
 	wmf_codec.MimeType = (const WCHAR*) wmf_mimetype;
-	wmf_codec.Flags = Decoder | SupportBitmap | Builtin;
+	wmf_codec.Flags = ImageCodecFlagsDecoder | ImageCodecFlagsSupportBitmap | ImageCodecFlagsBuiltin;
 	wmf_codec.Version = 1;
 	wmf_codec.SigCount = 1;
 	wmf_codec.SigSize = 4;
@@ -91,6 +92,12 @@ GetColor (WORD w1, WORD w2)
 	return color;
 }
 
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+#define GETW(x)		(*(WORD*)(data + (x)))
+#else
+#define GETW(x)		(GUINT16_FROM_LE(*(WORD*)(data + (x))))
+#endif
+
 #define GetParam(x,y)	GetWORD((6 + ((x) << 1)), (y))
 
 
@@ -117,8 +124,10 @@ Polygon (MetafilePlayContext *context, BYTE *data, int len)
 
 	int n = 2;
 	for (p = 0, pt = points; p < num; p++, pt++) {
-		pt->X = GETW(WP(n++));
-		pt->Y = GETW(WP(n++));
+		pt->X = GETW(WP(n));
+		n++;
+		pt->Y = GETW(WP(n));
+		n++;
 #ifdef DEBUG_WMF
 		printf ("\n\tpoly to %g,%g", pt->X, pt->Y);
 #endif
@@ -146,8 +155,10 @@ Polyline (MetafilePlayContext *context, BYTE *data)
 	SHORT y1 = GETW(WP3);
 	int n = 4;
 	for (p = 1; p < num; p++) {
-		SHORT x2 = GETW(WP(n++));
-		SHORT y2 = GETW(WP(n++));
+		SHORT x2 = GETW(WP(n));
+		n++;
+		SHORT y2 = GETW(WP(n));
+		n++;
 #ifdef DEBUG_WMF_2
 		printf ("\n\tdraw from %d,%d to %d,%d", x1, y1, x2, y2);
 #endif
@@ -179,7 +190,8 @@ PolyPolygon (MetafilePlayContext *context, BYTE *data)
 	int n = 2;
 	/* read size of each polygon and allocate the required memory */
 	for (i = 0; i < poly_num; i++) {
-		current->num = GETW(WP(n++));
+		current->num = GETW(WP(n));
+		n++;
 		current->points = (GpPointF*) GdipAlloc (current->num * sizeof (GpPointF));
 #ifdef DEBUG_WMF_2
 		printf ("\n\tSub Polygon #%d has %d points", i, current->num);
@@ -193,8 +205,10 @@ PolyPolygon (MetafilePlayContext *context, BYTE *data)
 		GpPointF *pt = current->points;
 		int p;
 		for (p = 0; p < current->num; p++) {
-			pt->X = GETW(WP(n++));
-			pt->Y = GETW(WP(n++));
+			pt->X = GETW(WP(n));
+			n++;
+			pt->Y = GETW(WP(n));
+			n++;
 #ifdef DEBUG_WMF_3
 			printf ("\n\t\tpoly to %g,%g", pt->X, pt->Y);
 #endif
