@@ -23,11 +23,9 @@
 #ifndef __WMFCODEC_H__
 #define __WMFCODEC_H__
 
-#include <stdio.h>
-#include "gdip.h"
-#include "gdipImage.h"
-#include "dstream.h"
-#include "metafile.h"
+#include "gdiplus-private.h"
+#include "graphics-private.h"
+#include "metafile-private.h"
 
 #define WMF_MIN_RECORD_SIZE	6
 
@@ -35,8 +33,6 @@
 		if (params < (x)) goto cleanup; \
 	} while (0)
 
-#define GETDW(x)	(*(DWORD*)(data + (x)))
-#define GETW(x)		(*(WORD*)(data + (x)))
 #define RECORDSIZE	0
 #define FUNCTION	4
 #define WP1		6
